# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: config.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0c\x63onfig.proto\x12\x1atykevision.agent.config.v1\x1a\x1egoogle/protobuf/wrappers.proto\"\x86\x04\n\x0b\x41gentConfig\x12\x32\n\x0cservice_name\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x38\n\treporting\x18\x02 \x01(\x0b\x32%.tykevision.agent.config.v1.Reporting\x12=\n\x0c\x64\x61ta_capture\x18\x03 \x01(\x0b\x32\'.tykevision.agent.config.v1.DataCapture\x12J\n\x13propagation_formats\x18\x04 \x03(\x0e\x32-.tykevision.agent.config.v1.PropagationFormat\x12+\n\x07\x65nabled\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x38\n\tjavaagent\x18\x06 \x01(\x0b\x32%.tykevision.agent.config.v1.JavaAgent\x12\\\n\x13resource_attributes\x18\x07 \x03(\x0b\x32?.tykevision.agent.config.v1.AgentConfig.ResourceAttributesEntry\x1a\x39\n\x17ResourceAttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x96\x03\n\tReporting\x12.\n\x08\x65ndpoint\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12*\n\x06secure\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12+\n\x05token\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12J\n\x13trace_reporter_type\x18\x05 \x01(\x0e\x32-.tykevision.agent.config.v1.TraceReporterType\x12/\n\tcert_file\x18\x06 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x35\n\x0fmetric_endpoint\x18\x07 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12L\n\x14metric_reporter_type\x18\x08 \x01(\x0e\x32..tykevision.agent.config.v1.MetricReporterType\"d\n\x07Message\x12+\n\x07request\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12,\n\x08response\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\"\xf1\x02\n\x0b\x44\x61taCapture\x12\x39\n\x0chttp_headers\x18\x01 \x01(\x0b\x32#.tykevision.agent.config.v1.Message\x12\x36\n\thttp_body\x18\x02 \x01(\x0b\x32#.tykevision.agent.config.v1.Message\x12\x39\n\x0crpc_metadata\x18\x03 \x01(\x0b\x32#.tykevision.agent.config.v1.Message\x12\x35\n\x08rpc_body\x18\x04 \x01(\x0b\x32#.tykevision.agent.config.v1.Message\x12\x38\n\x13\x62ody_max_size_bytes\x18\x05 \x01(\x0b\x32\x1b.google.protobuf.Int32Value\x12\x43\n\x1e\x62ody_max_processing_size_bytes\x18\x06 \x01(\x0b\x32\x1b.google.protobuf.Int32Value\"C\n\tJavaAgent\x12\x36\n\x10\x66ilter_jar_paths\x18\x01 \x03(\x0b\x32\x1c.google.protobuf.StringValue*-\n\x11PropagationFormat\x12\x06\n\x02\x42\x33\x10\x00\x12\x10\n\x0cTRACECONTEXT\x10\x01*`\n\x11TraceReporterType\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\n\n\x06ZIPKIN\x10\x01\x12\x08\n\x04OTLP\x10\x02\x12\x0b\n\x07LOGGING\x10\x03\x12\x08\n\x04NONE\x10\x04\x12\r\n\tOTLP_HTTP\x10\x05*\xbf\x01\n\x12MetricReporterType\x12$\n METRIC_REPORTER_TYPE_UNSPECIFIED\x10\x00\x12\x1d\n\x19METRIC_REPORTER_TYPE_OTLP\x10\x01\x12#\n\x1fMETRIC_REPORTER_TYPE_PROMETHEUS\x10\x02\x12 \n\x1cMETRIC_REPORTER_TYPE_LOGGING\x10\x03\x12\x1d\n\x19METRIC_REPORTER_TYPE_NONE\x10\x04\x42 \n\x1eorg.tykevision.agent.config.v1b\x06proto3')

_PROPAGATIONFORMAT = DESCRIPTOR.enum_types_by_name['PropagationFormat']
PropagationFormat = enum_type_wrapper.EnumTypeWrapper(_PROPAGATIONFORMAT)
_TRACEREPORTERTYPE = DESCRIPTOR.enum_types_by_name['TraceReporterType']
TraceReporterType = enum_type_wrapper.EnumTypeWrapper(_TRACEREPORTERTYPE)
_METRICREPORTERTYPE = DESCRIPTOR.enum_types_by_name['MetricReporterType']
MetricReporterType = enum_type_wrapper.EnumTypeWrapper(_METRICREPORTERTYPE)
B3 = 0
TRACECONTEXT = 1
UNSPECIFIED = 0
ZIPKIN = 1
OTLP = 2
LOGGING = 3
NONE = 4
OTLP_HTTP = 5
METRIC_REPORTER_TYPE_UNSPECIFIED = 0
METRIC_REPORTER_TYPE_OTLP = 1
METRIC_REPORTER_TYPE_PROMETHEUS = 2
METRIC_REPORTER_TYPE_LOGGING = 3
METRIC_REPORTER_TYPE_NONE = 4


_AGENTCONFIG = DESCRIPTOR.message_types_by_name['AgentConfig']
_AGENTCONFIG_RESOURCEATTRIBUTESENTRY = _AGENTCONFIG.nested_types_by_name['ResourceAttributesEntry']
_REPORTING = DESCRIPTOR.message_types_by_name['Reporting']
_MESSAGE = DESCRIPTOR.message_types_by_name['Message']
_DATACAPTURE = DESCRIPTOR.message_types_by_name['DataCapture']
_JAVAAGENT = DESCRIPTOR.message_types_by_name['JavaAgent']
AgentConfig = _reflection.GeneratedProtocolMessageType('AgentConfig', (_message.Message,), {

  'ResourceAttributesEntry' : _reflection.GeneratedProtocolMessageType('ResourceAttributesEntry', (_message.Message,), {
    'DESCRIPTOR' : _AGENTCONFIG_RESOURCEATTRIBUTESENTRY,
    '__module__' : 'config_pb2'
    # @@protoc_insertion_point(class_scope:tykevision.agent.config.v1.AgentConfig.ResourceAttributesEntry)
    })
  ,
  'DESCRIPTOR' : _AGENTCONFIG,
  '__module__' : 'config_pb2'
  # @@protoc_insertion_point(class_scope:tykevision.agent.config.v1.AgentConfig)
  })
_sym_db.RegisterMessage(AgentConfig)
_sym_db.RegisterMessage(AgentConfig.ResourceAttributesEntry)

Reporting = _reflection.GeneratedProtocolMessageType('Reporting', (_message.Message,), {
  'DESCRIPTOR' : _REPORTING,
  '__module__' : 'config_pb2'
  # @@protoc_insertion_point(class_scope:tykevision.agent.config.v1.Reporting)
  })
_sym_db.RegisterMessage(Reporting)

Message = _reflection.GeneratedProtocolMessageType('Message', (_message.Message,), {
  'DESCRIPTOR' : _MESSAGE,
  '__module__' : 'config_pb2'
  # @@protoc_insertion_point(class_scope:tykevision.agent.config.v1.Message)
  })
_sym_db.RegisterMessage(Message)

DataCapture = _reflection.GeneratedProtocolMessageType('DataCapture', (_message.Message,), {
  'DESCRIPTOR' : _DATACAPTURE,
  '__module__' : 'config_pb2'
  # @@protoc_insertion_point(class_scope:tykevision.agent.config.v1.DataCapture)
  })
_sym_db.RegisterMessage(DataCapture)

JavaAgent = _reflection.GeneratedProtocolMessageType('JavaAgent', (_message.Message,), {
  'DESCRIPTOR' : _JAVAAGENT,
  '__module__' : 'config_pb2'
  # @@protoc_insertion_point(class_scope:tykevision.agent.config.v1.JavaAgent)
  })
_sym_db.RegisterMessage(JavaAgent)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\036org.tykevision.agent.config.v1'
  _AGENTCONFIG_RESOURCEATTRIBUTESENTRY._options = None
  _AGENTCONFIG_RESOURCEATTRIBUTESENTRY._serialized_options = b'8\001'
  _PROPAGATIONFORMAT._serialized_start=1549
  _PROPAGATIONFORMAT._serialized_end=1594
  _TRACEREPORTERTYPE._serialized_start=1596
  _TRACEREPORTERTYPE._serialized_end=1692
  _METRICREPORTERTYPE._serialized_start=1695
  _METRICREPORTERTYPE._serialized_end=1886
  _AGENTCONFIG._serialized_start=77
  _AGENTCONFIG._serialized_end=595
  _AGENTCONFIG_RESOURCEATTRIBUTESENTRY._serialized_start=538
  _AGENTCONFIG_RESOURCEATTRIBUTESENTRY._serialized_end=595
  _REPORTING._serialized_start=598
  _REPORTING._serialized_end=1004
  _MESSAGE._serialized_start=1006
  _MESSAGE._serialized_end=1106
  _DATACAPTURE._serialized_start=1109
  _DATACAPTURE._serialized_end=1478
  _JAVAAGENT._serialized_start=1480
  _JAVAAGENT._serialized_end=1547
# @@protoc_insertion_point(module_scope)
